@@ -12,6 +12,7 @@ import urllib2
 import urlparse
 from string import Template
 from pyactiveresource import connection
+from pyactiveresource import element_containers
 from pyactiveresource import formats
 from pyactiveresource import util
 
@@ -85,7 +86,7 @@ class Errors(object):
         attribute_keys = self.base.attributes.keys()
         try:
             messages = util.xml_to_dict(
-                    xml_string, saveroot=True)['errors']['error']
+                    xml_string)['errors']['error']
             if not isinstance(messages, list):
                 messages = [messages]
         except util.Error:
@@ -159,10 +160,7 @@ class ResourceMeta(type):
             new_attrs: The class attribute dictionary.
         """
         if '_singular' not in new_attrs or not new_attrs['_singular']:
-            # Convert CamelCase to lower_underscore
-            singular = re.sub(r'\B((?<=[a-z])[A-Z]|[A-Z](?=[a-z]))',
-                              r'_\1', name).lower()
-            new_attrs['_singular'] = singular
+            new_attrs['_singular'] = util.underscore(name)
 
         if '_plural' not in new_attrs or not new_attrs['_plural']:
             new_attrs['_plural'] = util.pluralize(new_attrs['_singular'])
@@ -249,6 +247,7 @@ class ResourceMeta(type):
         return cls._format
 
     def set_format(cls, value):
+        cls._connection = None
         cls._format = value
     format = property(get_format, set_format, None,
                        'A format object for encoding/decoding requests')
@@ -279,7 +278,7 @@ class ResourceMeta(type):
     def set_prefix_source(cls, value):
         """Set the prefix source, which will be rendered into the prefix."""
         cls._prefix_source = value
-    
+
     prefix_source = property(get_prefix_source, set_prefix_source, None,
                              'prefix for lookups for this type of object.')
 
@@ -287,6 +286,15 @@ class ResourceMeta(type):
         """Return the rendered prefix for this object."""
         return cls._prefix(options)
 
+    def get_primary_key(cls):
+        return cls._primary_key
+
+    def set_primary_key(cls, value):
+        cls._primary_key = value
+
+    primary_key = property(get_primary_key, set_primary_key, None,
+                           'Name of attribute that uniquely identies the resource')
+
 
 class ActiveResource(object):
     """Represents an activeresource object."""
@@ -299,6 +307,7 @@ class ActiveResource(object):
     _site = None
     _timeout = None
     _user = None
+    _primary_key = "id"
 
     def __init__(self, attributes=None, prefix_options=None):
         """Initialize a new ActiveResource object.
@@ -400,7 +409,7 @@ class ActiveResource(object):
         Args:
             attributes: A dictionary of attributes which represent this object.
         Returns:
-            True if the resource is found, False otherwise.
+            The new resource (which may or may not have been saved successfully).
         """
         resource = cls(attributes)
         resource.save()
@@ -419,7 +428,7 @@ class ActiveResource(object):
         #TODO(mrroach): figure out prefix_options
         prefix_options = {}
         query_options = {}
-        for key, value in options.items():
+        for key, value in options.iteritems():
             if key in cls._prefix_parameters():
                 prefix_options[key] = value
             else:
@@ -469,11 +478,13 @@ class ActiveResource(object):
         Returns:
             A list of resources.
         """
+        prefix_options, query_options = cls._split_options(kwargs)
         if from_:
-            path = from_ + cls._query_string(kwargs)
+            query_options.update(prefix_options)
+            path = from_ + cls._query_string(query_options)
             prefix_options = None
         else:
-            prefix_options, query_options = cls._split_options(kwargs)
+
             path = cls._collection_path(prefix_options, query_options)
         return cls._build_list(cls.connection.get(path, cls.headers),
                                prefix_options)
@@ -503,6 +514,8 @@ class ActiveResource(object):
             A list of ActiveResource objects.
         """
         resources = []
+        if isinstance(elements, dict):
+          elements = [elements]
         # slice elements to ensure that this is a list-type object not a dict
         for element in elements[:]:
             resources.append(cls(element, prefix_options))
@@ -518,7 +531,7 @@ class ActiveResource(object):
             A string containing the encoded query.
         """
         if query_options:
-            return '?' + urllib.urlencode(query_options)
+            return '?' + util.to_query(query_options)
         else:
             return ''
 
@@ -617,10 +630,10 @@ class ActiveResource(object):
 
     @classmethod
     def _class_put(cls, method_name, body='', **kwargs):
-        """Get a nested resource or resources.
+        """Update a nested resource or resources.
 
         Args:
-            method_name: the nested resource to retrieve.
+            method_name: the nested resource to update.
             body: The data to send as the body of the request.
             kwargs: Any keyword arguments for the query.
         Returns:
@@ -631,10 +644,10 @@ class ActiveResource(object):
 
     @classmethod
     def _class_delete(cls, method_name, **kwargs):
-        """Get a nested resource or resources.
+        """Delete a nested resource or resources.
 
         Args:
-            method_name: the nested resource to retrieve.
+            method_name: the nested resource to delete.
             kwargs: Any keyword arguments for the query.
         Returns:
             A connection.Response object.
@@ -642,6 +655,19 @@ class ActiveResource(object):
         url = cls._custom_method_collection_url(method_name, kwargs)
         return cls.connection.delete(url, cls.headers)
 
+    @classmethod
+    def _class_head(cls, method_name, **kwargs):
+        """Predicate a nested resource or resources exists.
+
+        Args:
+            method_name: the nested resource to predicate exists.
+            kwargs: Any keyword arguments for the query.
+        Returns:
+            A connection.Response object.
+        """
+        url = cls._custom_method_collection_url(method_name, kwargs)
+        return cls.connection.head(url, cls.headers)
+
     @classmethod
     def _prefix_parameters(cls):
         """Return a list of the parameters used in the site prefix.
@@ -686,27 +712,48 @@ class ActiveResource(object):
         values = {}
         for key, value in self.attributes.iteritems():
             if isinstance(value, list):
-                values[key] = [i.to_dict() for i in value]
+                new_value = []
+                for i in value:
+                  if isinstance(i, dict):
+                      new_value.append(i)
+                  else:
+                      new_value.append(i.to_dict())
+                values[key] = new_value
             elif isinstance(value, ActiveResource):
                 values[key] = value.to_dict()
             else:
                 values[key] = value
         return values
 
-    def to_xml(self, root=None, header=True, pretty=False):
+    def to_xml(self, root=None, header=True, pretty=False, dasherize=True):
         """Convert the object to an xml string.
 
         Args:
             root: The name of the root element for xml output.
             header: Whether to include the xml header.
             pretty: Whether to "pretty-print" format the output.
+            dasherize: Whether to dasherize the xml attribute names.
         Returns:
             An xml string.
         """
         if not root:
             root = self._singular
         return util.to_xml(self.to_dict(), root=root,
-                           header=header, pretty=pretty)
+                           header=header, pretty=pretty,
+                           dasherize=dasherize)
+
+    def reload(self):
+        """Connect to the server and update this resource's attributes.
+
+        Args:
+            None
+        Returns:
+            None
+        """
+        attributes = self.klass.connection.get(
+                self._element_path(self.id, self._prefix_options),
+                self.klass.headers)
+        self._update(attributes)
 
     def save(self):
         """Save the object to the server.
@@ -720,6 +767,7 @@ class ActiveResource(object):
             connection.Error: On any communications problems.
         """
         try:
+            self.errors.clear()
             if self.id:
                 response = self.klass.connection.put(
                         self._element_path(self.id, self._prefix_options),
@@ -732,7 +780,7 @@ class ActiveResource(object):
                         data=self.to_xml())
                 new_id = self._id_from_response(response)
                 if new_id:
-                    self.attributes['id'] = new_id
+                    self.id = new_id
         except connection.ResourceInvalid, err:
             self.errors.from_xml(err.response.body)
             return False
@@ -783,6 +831,14 @@ class ActiveResource(object):
                 self._element_path(self.id, self._prefix_options),
                 self.klass.headers)
 
+    def get_id(self):
+        return self.attributes.get(self.klass.primary_key)
+
+    def set_id(self, value):
+        self.attributes[self.klass.primary_key] = value
+
+    id = property(get_id, set_id, None, 'Value stored in the primary key')
+
     def __getattr__(self, name):
         """Retrieve the requested attribute if it exists.
 
@@ -793,12 +849,9 @@ class ActiveResource(object):
         Raises:
             AttributeError: if no such attribute exists.
         """
-        #TODO(mrroach): Use descriptors instead of __getattr__
-        if name == 'id':
-            # id should always be getattrable
-            return self.attributes.get('id')
-        if name in self.attributes:
-            return self.attributes[name]
+        if 'attributes' in self.__dict__:
+            if name in self.attributes:
+                return self.attributes[name]
         raise AttributeError(name)
 
     def __setattr__(self, name, value):
@@ -811,7 +864,7 @@ class ActiveResource(object):
             None
         """
         if '_initialized' in self.__dict__:
-            if name in self.__dict__ or name in self.__class__.__dict__:
+            if name in self.__dict__ or getattr(self.__class__, name, None):
                 # Update a normal attribute
                 object.__setattr__(self, name, value)
             else:
@@ -829,6 +882,9 @@ class ActiveResource(object):
         else:
             return cmp(self.id, other)
 
+    def __hash__(self):
+        return hash(tuple(self.attributes.items()))
+
     def _update(self, attributes):
         """Update the object with the given attributes.
 
@@ -844,21 +900,38 @@ class ActiveResource(object):
                 klass = self._find_class_for(key)
                 attr = klass(value)
             elif isinstance(value, list):
-                klass = self._find_class_for(util.singularize(key))
+                klass = self._find_class_for_collection(key)
                 attr = [klass(child) for child in value]
             else:
                 attr = value
             # Store the actual value in the attributes dictionary
             self.attributes[key] = attr
 
-    def _find_class_for(self, element_name=None, class_name=None):
+    @classmethod
+    def _find_class_for_collection(cls, collection_name):
         """Look in the parent modules for classes matching the element name.
 
-        One, or both of element/class name must be specified.
+        One or both of element/class name must be specified.
+
+        Args:
+            collection_name: The name of the collection type.
+        Returns:
+            A Resource class.
+        """
+        return cls._find_class_for(util.singularize(collection_name))
+
+    @classmethod
+    def _find_class_for(cls, element_name=None,
+                        class_name=None, create_missing=True):
+        """Look in the parent modules for classes matching the element name.
+
+        One or both of element/class name must be specified.
 
         Args:
             element_name: The name of the element type.
             class_name: The class name of the element type.
+            create_missing: Whether classes should be auto-created if no
+                existing match is found.
         Returns:
             A Resource class.
         """
@@ -869,7 +942,7 @@ class ActiveResource(object):
         elif not class_name:
             class_name = util.camelize(element_name)
 
-        module_path = self.__module__.split('.')
+        module_path = cls.__module__.split('.')
         for depth in range(len(module_path), 0, -1):
             try:
                 __import__('.'.join(module_path[:depth]))
@@ -893,8 +966,9 @@ class ActiveResource(object):
                     continue
 
         # If we made it this far, no such class was found
-        return new.classobj(class_name, (self.__class__,),
-                            {'__module__': self.__module__})
+        if create_missing:
+            return new.classobj(class_name, (cls,),
+                                {'__module__': cls.__module__})
 
     # methods corresponding to Ruby's custom_methods
     def _custom_method_element_url(self, method_name, options):
@@ -982,10 +1056,10 @@ class ActiveResource(object):
         return self.klass.connection.put(url, self.klass.headers, body)
 
     def _instance_delete(self, method_name, **kwargs):
-        """Get a nested resource or resources.
+        """Delete a nested resource or resources.
 
         Args:
-            method_name: the nested resource to retrieve.
+            method_name: the nested resource to delete.
             kwargs: Any keyword arguments for the query.
         Returns:
             A connection.Response object.
@@ -993,8 +1067,21 @@ class ActiveResource(object):
         url = self._custom_method_element_url(method_name, kwargs)
         return self.klass.connection.delete(url, self.klass.headers)
 
+    def _instance_head(self, method_name, **kwargs):
+        """Predicate a nested resource or resources exists.
+
+        Args:
+            method_name: the nested resource to predicate exists.
+            kwargs: Any keyword arguments for the query.
+        Returns:
+            A connection.Response object.
+        """
+        url = self._custom_method_element_url(method_name, kwargs)
+        return self.klass.connection.head(url, self.klass.headers)
+
     # Create property which returns class/instance method based on context
     get = ClassAndInstanceMethod('_class_get', '_instance_get')
     post = ClassAndInstanceMethod('_class_post', '_instance_post')
     put = ClassAndInstanceMethod('_class_put', '_instance_put')
     delete = ClassAndInstanceMethod('_class_delete', '_instance_delete')
+    head = ClassAndInstanceMethod('_class_head', '_instance_head')
