@@ -5,7 +5,7 @@
 
 __author__ = 'Mark Roach (mrroach@google.com)'
 
-
+import logging
 from pyactiveresource import util
 
 
@@ -26,11 +26,12 @@ class XMLFormat(Base):
     @staticmethod
     def decode(resource_string):
         """Convert a resource string to a dictionary."""
+        log = logging.getLogger('pyactiveresource.format')
+        log.debug('decoding resource: %s', resource_string)
         try:
-            data = util.xml_to_dict(resource_string, saveroot=True)
+            data = util.xml_to_dict(resource_string, saveroot=False)
         except util.Error, err:
             raise Error(err)
         if isinstance(data, dict) and len(data) == 1:
             data = data.values()[0]
         return data
-
